from os.path import dirname, join
from plone.app.imaging import tests


def getData(filename):
    """ return contents of the file with the given name """
    filename = join(dirname(tests.__file__), filename)
    return open(filename, 'r').read()



from unittest import defaultTestLoader
from Products.Archetypes.interfaces import IImageField
from Products.ATContentTypes.content.image import ATImageSchema, ATImage
from plone.app.imaging.tests.base import ImagingTestCase
from plone.app.imaging.monkey import getAvailableSizes
from plone.app.imaging.tests.base import getSettings


class MonkeyPatchTests(ImagingTestCase):

    def afterSetUp(self):
        self.image = ATImage('test')
        self.field = self.image.getField('image')
        self.sizes = ATImageSchema['image'].sizes   # save original value

    def beforeTearDown(self):
        ATImageSchema['image'].sizes = self.sizes   # restore original value

    def testAvailableSizes(self):
        # make sure the field was patched
        self.assertEqual(self.field.getAvailableSizes.func_code,
                         getAvailableSizes.func_code)
        # set custom image sizes and check the helper
        settings = getSettings()
        settings.allowed_sizes = [u'foo 23:23', u'bar 8:8']
        self.assertEqual(self.field.getAvailableSizes(self.image),
                         dict(foo=(23, 23), bar=(8, 8)))

    def testAvailableSizesInstanceMethod(self):
        marker = dict(foo=23)
        def foo(self):
            return marker
        ATImage.foo = foo                       # create new instance method
        ATImageSchema['image'].sizes = 'foo'    # restore original value
        self.assertEqual(self.field.getAvailableSizes(self.image), marker)

    def testAvailableSizesCallable(self):
        def foo():
            return 'foo!'
        ATImageSchema['image'].sizes = foo      # store method in schema
        self.assertEqual(self.field.getAvailableSizes(self.image), 'foo!')

    def testAvailableSizesOnField(self):
        marker = dict(foo=23)
        ATImageSchema['image'].sizes = marker   # store dict in schema
        self.assertEqual(self.field.getAvailableSizes(self.image), marker)


class RegistryTests(ImagingTestCase):

    def testImageFieldInterface(self):
        data = self.getImage()
        folder = self.folder
        image = folder[folder.invokeFactory('Image', id='foo', image=data)]
        field = image.getField('image')
        self.assertTrue(IImageField.providedBy(field))


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)



from zope.interface import implements
from Products.Archetypes.Field import Image
from plone.app.imaging.interfaces import IImageScale


class ImageScale(Image):
    """ extend image class from `Archetypes.Field` by making sure the title
        gets always computed and not calling `_get_content_type` even though
        an explicit type has been passed """
    implements(IImageScale)

    def __init__(self, id, data, content_type, **kw):
        self.__name__ = id
        self.__dict__.update(kw)
        self.precondition = ''


        # `OFS.Image` has no proper support for file objects or iterators,
        # so we'll require `data` to be a string or a file-like object...
        if not isinstance(data, str):
            data = data.open('r').read()    # assume it's file-like
        self.update_data(data, content_type, size=len(data))

    def absolute_url(self):
        """ return the url for new-style scales, but fall back for others """
        if 'url' in self.__dict__:
            return self.url
        else:
            return super(ImageScale, self).absolute_url()

    def __call__(self, *args, **kw):
        """ calling the scale returns itself, so "nocall:" can be skipped """
        return self



from plone.app.testing.bbb import PloneTestCase
from plone.testing.z2 import Browser
from plone.app.imaging import testing
from StringIO import StringIO
from os.path import dirname, join
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from zope.component import queryUtility
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces.controlpanel import IImagingSchema


def getSettings():
    registry = queryUtility(IRegistry)
    return registry.forInterface(IImagingSchema, prefix="plone", check=False)


def getData(filename):
    """ return contents of the file with the given name """
    filename = join(dirname(__file__), filename)
    return open(filename, 'r').read()


class ImagingTestCase(PloneTestCase):
    """ base class for integration tests """

    layer = testing.imaging

    def getImage(self, name='image.png'):
        return getData(name)

    def assertImage(self, data, format, size):
        import PIL.Image
        image = PIL.Image.open(StringIO(data))
        self.assertEqual(image.format, format)
        self.assertEqual(image.size, size)


class ImagingFunctionalTestCase(ImagingTestCase):
    """ base class for functional tests """

    def getCredentials(self):
        return '%s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD)

    def getBrowser(self, loggedIn=True):
        """ instantiate and return a testbrowser for convenience """
        browser = Browser(self.layer['app'])
        if loggedIn:
            auth = 'Basic %s' % self.getCredentials()
            browser.addHeader('Authorization', auth)
        return browser



from Products.CMFCore.permissions import setDefaultRoles
try:
	import Products.ATContentTypes
except ImportError:
	HAS_ATCT = False
else:
	HAS_ATCT = True
	from plone.app.imaging.monkey import patchImageField
	from plone.app.imaging.monkey import patchSchemas

setDefaultRoles('Plone Site Setup: Imaging', ('Manager', 'Site Administrator'))

def initialize(context):
	if HAS_ATCT:
	    patchImageField()       # patch ImageField's `getAvailableSizes` method
	    patchSchemas()          # patch ATCT schemas with `sizes` attribute



from plone.app.imaging.tests.base import ImagingTestCase
from plone.app.imaging.utils import getAllowedSizes, getQuality
from unittest import defaultTestLoader
from plone.app.imaging.tests.base import getSettings


class PropertiesTests(ImagingTestCase):

    def testAllowedSizes(self):
        # test the defaults
        # for readability, pep8 is not applied to the dict below
        self.assertEqual(getAllowedSizes(), dict(
            large=(768, 768),
            preview=(400, 400),
            mini=(200, 200),
            thumb=(128, 128),
            tile=(64, 64),
            icon=(32, 32),
            listing=(16, 16)))
        # override and test again
        settings = getSettings()
        settings.allowed_sizes = [u'foo bar 23:23']
        self.assertEqual(getAllowedSizes(), dict(foo_bar=(23, 23)))

    def testQuality(self):
        self.assertEqual(getQuality(), 88)
        settings = getSettings()
        settings.quality = 42
        self.assertEqual(getQuality(), 42)

def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)



from logging import exception
from zope.component import adapts
from zope.interface import implements
from zope.publisher.interfaces import IRequest
from Products.Archetypes.interfaces import IImageField
from Products.Archetypes.Field import HAS_PIL
from ZODB.POSException import ConflictError
from ZPublisher.BaseRequest import DefaultPublishTraverse
from plone.app.imaging.interfaces import IBaseObject
from plone.app.imaging.interfaces import IImageScaleHandler
from plone.app.imaging.scale import ImageScale
from plone.protect.interfaces import IDisableCSRFProtection
from zope.interface import alsoProvides
from zope.globalrequest import getRequest


class ImageTraverser(DefaultPublishTraverse):
    """ traversal adapter for scaled down versions of image content """
    adapts(IBaseObject, IRequest)

    def fallback(self, request, name):
        return super(ImageTraverser, self).publishTraverse(request, name)

    def publishTraverse(self, request, name):
        schema = self.context.Schema()
        if '_' in name:
            fieldname, scale = name.split('_', 1)
        else:
            fieldname, scale = name, None
        field = schema.get(fieldname)
        handler = IImageScaleHandler(field, None)
        if handler is not None:
            image = handler.getScale(self.context, scale)
            if image is not None:
                return image
        return self.fallback(request, name)


class DefaultImageScaleHandler(object):
    """ default handler for creating and storing scaled version of images """
    implements(IImageScaleHandler)
    adapts(IImageField)

    def __init__(self, context):
        self.context = context

    def getScale(self, instance, scale):
        """ return scaled and aq-wrapped version for given image data """
        field = self.context
        available = field.getAvailableSizes(instance)
        if scale in available or scale is None:
            image = self.retrieveScale(instance, scale=scale)
            if not image:       # create the scale if it doesn't exist
                width, height = available[scale]
                data = self.createScale(instance, scale, width, height)
                if data is not None:
                    self.storeScale(instance, scale, **data)
                    image = self.retrieveScale(instance, scale=scale)
            if image is not None and not isinstance(image, basestring):
                return image
        return None

    def createScale(self, instance, scale, width, height, data=None):
        """ create & return a scaled version of the image as retrieved
            from the field or optionally given data """
        # disable CRSF on scale generation
        req = getRequest()
        if req:
            alsoProvides(req, IDisableCSRFProtection)
        field = self.context
        if HAS_PIL and width and height:
            if data is None:
                image = field.getRaw(instance)
                if not image:
                    return None
                data = str(image.data)
            if data:
                id = field.getName() + '_' + scale
                try:
                    imgdata, format = field.scale(data, width, height)
                except (ConflictError, KeyboardInterrupt):
                    raise
                except Exception:
                    if not field.swallowResizeExceptions:
                        raise
                    else:
                        exception('could not scale ImageField "%s" of %s',
                            field.getName(), instance.absolute_url())
                        return None
                content_type = 'image/%s' % format.lower()
                filename = field.getFilename(instance)
                return dict(id=id, data=imgdata.getvalue(),
                    content_type=content_type, filename=filename)
        return None

    def retrieveScale(self, instance, scale):
        """ retrieve a scaled version of the image """
        field = self.context
        return field.getScale(instance, scale=scale)

    def storeScale(self, instance, scale, **data):
        """ store a scaled version of the image """
        image = ImageScale(**data)
        field = self.context
        field.getStorage(instance).set(image.getId(), instance, image,
            mimetype=image.content_type, filename=image.filename)



from plone.app.imaging.tests.base import ImagingTestCase
from plone.app.imaging.tests.base import ImagingFunctionalTestCase
from plone.app.imaging.tests.base import getSettings
from plone.app.imaging.scaling import ImageScaling
from re import match
from unittest import defaultTestLoader

import transaction


class ImageStandardTraverseTests(ImagingTestCase):
    # Note: this class is subclassed by ImageChameleonTraverseTests, which
    # inherits our tests but uses a different traverser.  We use the standard
    # Zope pagetemplate traverser.

    def traverser(self, view, path=''):
        # Standard Zope page template traversal uses a list as path.
        # This is a simplified version specialised for the scaling view.
        stack = path.split('/')
        while stack:
            name = stack.pop(0)
            view = view.traverse(name, stack)
        return view

    def afterSetUp(self):
        self.data = self.getImage()
        self.image = self.folder[self.folder.invokeFactory(
            'Image', id='foo', image=self.data)]
        field = self.image.getField('image')
        self.available = field.getAvailableSizes(self.image)

    def traverse(self, path=''):
        view = self.image.unrestrictedTraverse('@@images')
        tag = self.traverser(view, path)
        base = self.image.absolute_url()
        expected = r'<img src="%s/@@images/([-0-9a-f]{36}).(jpeg|gif|png)" ' \
            r'alt="foo" title="foo" height="(\d+)" width="(\d+)" />' % base
        groups = match(expected, tag).groups()
        self.assertTrue(groups, tag)
        uid, ext, height, width = groups
        return uid, ext, int(width), int(height)

    def testImageThumb(self):
        self.assertTrue('thumb' in self.available.keys())
        uid, ext, width, height = self.traverse('image/thumb')
        self.assertEqual((width, height), self.available['thumb'])
        self.assertEqual(ext, 'png')

    def testCustomSizes(self):
        # set custom image sizes
        settings = getSettings()
        settings.allowed_sizes = [u'foo 23:23']
        # make sure traversing works with the new sizes
        uid, ext, width, height = self.traverse('image/foo')
        self.assertEqual(width, 23)
        self.assertEqual(height, 23)

    def testScaleInvalidation(self):
        # first view the thumbnail of the original image
        uid1, ext, width1, height1 = self.traverse('image/thumb')
        # now upload a new one and make sure the thumbnail has changed
        self.image.update(image=self.getImage('image.jpg'))
        uid2, ext, width2, height2 = self.traverse('image/thumb')
        self.assertNotEqual(uid1, uid2, 'thumb not updated?')
        # the height also differs as the original image had a size of 200, 200
        # whereas the updated one has 500, 200...
        self.assertEqual(width1, width2)
        self.assertNotEqual(height1, height2)

    def testCustomSizeChange(self):
        # set custom image sizes & view a scale
        settings = getSettings()
        settings.allowed_sizes = [u'foo 23:23']
        uid1, ext, width, height = self.traverse('image/foo')
        self.assertEqual(width, 23)
        self.assertEqual(height, 23)
        # now let's update the scale dimensions, after which the scale
        # should also have been updated...
        settings.allowed_sizes = [u'foo 42:42']
        uid2, ext, width, height = self.traverse('image/foo')
        self.assertEqual(width, 42)
        self.assertEqual(height, 42)
        self.assertNotEqual(uid1, uid2, 'scale not updated?')


class ImageChameleonTraverseTests(ImageStandardTraverseTests):
    # This class inherits all test methods from its parent, but uses the
    # Chameleon/five.pt traverser.

    def traverser(self, view, path=''):
        # five.pt/chameleon uses a tuple as path.  This is a simplified
        # version of BoboAwareZopeTraverse.traverse from five.pt.expressions,
        # specialised for the scaling view.
        path_items = tuple(path.split('/'))
        length = len(path_items)
        if length:
            i = 0
            while i < length:
                name = path_items[i]
                i += 1
                view = view.traverse(name, path_items[i:])
        return view


class ImageTagTests(ImagingTestCase):

    def afterSetUp(self):
        self.data = self.getImage()
        self.image = self.folder[self.folder.invokeFactory(
            'Image', id='foo', image=self.data)]
        field = self.image.getField('image')
        self.available = field.getAvailableSizes(self.image)

    def testViewTagMethod(self):
        folder = self.folder
        image = folder['foo']
        traverse = folder.REQUEST.traverseName
        view = traverse(image, '@@images')
        self.assertEqual(image.tag(), view.tag())

    def testViewTagMethodCustomScale(self):
        folder = self.folder
        image = folder['foo']
        traverse = folder.REQUEST.traverseName
        view = traverse(image, '@@images')
        view_tag = view.tag(width=23, height=23, alt="foo", title="foo")
        base = self.image.absolute_url()
        expected = r'<img src="%s/@@images/([-0-9a-f]{36}).(jpeg|gif|png)" ' \
            r'height="(\d+)" width="(\d+)" alt="foo" title="foo" />' % base
        name, ext, height, width = match(expected, view_tag).groups()
        self.assertEqual(height, '23')
        self.assertEqual(width, '23')
        scale = view.publishTraverse(self.image.REQUEST, name + "." + ext)
        self.assertEqual(scale.height, 23)
        self.assertEqual(scale.width, 23)


class ImagePublisherTests(ImagingFunctionalTestCase):

    def afterSetUp(self):
        data = self.getImage()
        folder = self.folder
        foo = folder[folder.invokeFactory('Image', id='foo', image=data)]
        self.view = foo.unrestrictedTraverse('@@images')
        transaction.commit()

    def testPublishScaleViaUID(self):
        scale = self.view.scale('image', width=64, height=64)
        # make sure the referenced image scale is available
        url = scale.url.replace('http://nohost', '')
        response = self.publish(url, basic=self.getCredentials())
        self.assertEqual(response.getStatus(), 200)
        self.assertEqual(response.getHeader('Content-Type'), 'image/png')
        self.assertImage(response.getBody(), 'PNG', (64, 64))

    def testPublishWebDavScaleViaUID(self):
        scale = self.view.scale('image', width=64, height=64)
        # make sure the referenced image scale is available
        url = scale.url.replace('http://nohost', '') + '/manage_DAVget'
        response = self.publish(url, basic=self.getCredentials())
        self.assertEqual(response.getStatus(), 200)
        # We get a very different response.  In the end it works out.
        self.assertTrue(
            'text/plain; charset=' in response.getHeader('Content-Type')
        )
        self.assertImage(response.getBody(), 'PNG', (64, 64))

    def testPublishFTPScaleViaUID(self):
        scale = self.view.scale('image', width=64, height=64)
        # make sure the referenced image scale is available
        url = scale.url.replace('http://nohost', '') + '/manage_FTPget'
        response = self.publish(url, basic=self.getCredentials())
        self.assertEqual(response.getStatus(), 200)
        # We get a very different response.  In the end it works out.
        self.assertTrue(
            'text/plain; charset=' in response.getHeader('Content-Type')
        )
        self.assertImage(response.getBody(), 'PNG', (64, 64))

    def testPublishThumbViaUID(self):
        scale = self.view.scale('image', 'thumb')
        # make sure the referenced image scale is available
        url = scale.url.replace('http://nohost', '')
        response = self.publish(url, basic=self.getCredentials())
        self.assertEqual(response.getStatus(), 200)
        self.assertEqual(response.getHeader('Content-Type'), 'image/png')
        self.assertImage(response.getBody(), 'PNG', (128, 128))

    def testPublishCustomSizeViaUID(self):
        # set custom image sizes
        settings = getSettings()
        settings.allowed_sizes = [u'foo 23:23']
        scale = self.view.scale('image', 'foo')
        # make sure the referenced image scale is available
        url = scale.url.replace('http://nohost', '')
        response = self.publish(url, basic=self.getCredentials())
        self.assertEqual(response.getStatus(), 200)
        self.assertEqual(response.getHeader('Content-Type'), 'image/png')
        self.assertImage(response.getBody(), 'PNG', (23, 23))

    def testPublishThumbViaName(self):
        # make sure traversing works as is and with scaling
        base = self.folder.absolute_url()
        browser = self.getBrowser()
        # first the field without a scale name
        browser.open(base + '/foo/@@images/image')
        self.assertEqual(browser.headers['status'], '200 Ok')
        self.assertEqual(browser.contents, self.getImage())
        self.assertEqual(browser.headers['Content-Type'], 'image/png')
        # and last a scaled version
        browser.open(base + '/foo/@@images/image/thumb')
        self.assertEqual(browser.headers['status'], '200 Ok')
        self.assertImage(browser.contents, 'PNG', (128, 128))
        self.assertEqual(browser.headers['Content-Type'], 'image/png')

    def testPublishCustomSizeViaName(self):
        # set custom image sizes
        settings = getSettings()
        settings.allowed_sizes = [u'foo 23:23']
        # make sure traversing works as expected
        base = '/'.join(self.folder.getPhysicalPath())
        credentials = self.getCredentials()
        response = self.publish(base + '/foo/@@images/image/foo',
                                basic=credentials)
        self.assertEqual(response.getStatus(), 200)
        self.assertImage(response.getBody(), 'PNG', (23, 23))

    def testPublishScaleWithInvalidUID(self):
        scale = self.view.scale('image', width=64, height=64)
        url = scale.url.replace('http://nohost', '')
        # change the url so it's invalid...
        url = url.replace('.png', 'x.png')
        response = self.publish(url, basic=self.getCredentials())
        self.assertEqual(response.getStatus(), 404)

    def testPublishScaleWithInvalidScale(self):
        scale = self.view.scale('image', 'no-such-scale')
        self.assertEqual(scale, None)

    def test_getAvailableSizesWithInvalidScale(self):
        self.assertEqual(self.view.getAvailableSizes('no-such-scale'), {})

    def test_getImageSizeWithInvalidScale(self):
        self.assertEqual(self.view.getImageSize('no-such-scale'), (0, 0))


class ScalesAdapterTests(ImagingTestCase):

    def afterSetUp(self):
        data = self.getImage()
        folder = self.folder
        self.image = folder[folder.invokeFactory(
            'Image', id='foo', image=data)]
        self.adapter = ImageScaling(self.image, None)
        self.settings = getSettings()
        self.settings.allowed_sizes = [u'foo 60:60']

    def testCreateScale(self):
        foo = self.adapter.scale('image', width=100, height=80)
        self.assertTrue(foo.uid)
        self.assertEqual(foo.mimetype, 'image/png')
        self.assertEqual(foo.width, 80)
        self.assertEqual(foo.height, 80)
        self.assertImage(foo.data, 'PNG', (80, 80))

    def testCreateScaleWithoutData(self):
        folder = self.folder
        image = folder[folder.invokeFactory('Image', id='image')]
        adapter = ImageScaling(image, None)
        foo = adapter.scale('image', width=100, height=80)
        self.assertEqual(foo, None)

    def testCreateScaleWithPdata(self):
        data = self.getImage() + '\x00' * (1 << 16)
        from Products.ATContentTypes.content.image import ATImage
        image = ATImage('image').__of__(self.folder)
        image.setImage(data)
        field = image.getField('image')
        field.swallowResizeExceptions = False
        from OFS.Image import Pdata
        self.assertTrue(isinstance(image.getImage().data, Pdata))
        adapter = ImageScaling(image, None)
        foo = adapter.scale('image', width=100, height=80)
        self.assertFalse(foo is None)

    def testGetScaleByName(self):
        foo = self.adapter.scale('image', scale='foo')
        self.assertTrue(foo.uid)
        self.assertEqual(foo.mimetype, 'image/png')
        self.assertEqual(foo.width, 60)
        self.assertEqual(foo.height, 60)
        self.assertImage(foo.data, 'PNG', (60, 60))

    def testGetUnknownScale(self):
        foo = self.adapter.scale('image', scale='foo?')
        self.assertEqual(foo, None)

    def testScaleInvalidation(self):
        # first get the scale of the original image
        foo1 = self.adapter.scale('image', scale='foo')
        # now upload a new one and make sure the scale has changed
        self.image.update(image=self.getImage('image.jpg'))
        foo2 = self.adapter.scale('image', scale='foo')
        self.assertFalse(foo1.data == foo2.data, 'scale not updated?')

    def testCustomSizeChange(self):
        # set custom image sizes & view a scale
        self.settings.allowed_sizes = [u'foo 23:23']
        foo = self.adapter.scale('image', scale='foo')
        self.assertEqual(foo.width, 23)
        self.assertEqual(foo.height, 23)
        # now let's update the scale dimensions, after which the scale
        # shouldn't be the same...
        self.settings.allowed_sizes = [u'foo 42:42']
        foo = self.adapter.scale('image', scale='foo')
        self.assertEqual(foo.width, 42)
        self.assertEqual(foo.height, 42)

    def testQualityChange(self):
        settings = getSettings()
        self.image.update(image=self.getImage('image.jpg'))
        data = self.getImage('image.jpg') + '\x00' * (1 << 16)
        # get size of image scaled at default scaling quality
        self.assertEqual(settings.quality, 88)
        from Products.ATContentTypes.content.image import ATImage
        image = ATImage('image').__of__(self.folder)
        image.setImage(data)
        adapter = ImageScaling(image, None)
        img_high_quality = adapter.scale('image', width=100, height=80)
        size_high_quality = img_high_quality.size
        # lower scaling quality and get scaled image's size at that quality
        settings.quality = 20
        self.assertEqual(settings.quality, 20)
        image = ATImage('image').__of__(self.folder)
        image.setImage(data)
        adapter = ImageScaling(image, None)
        img_low_quality = adapter.scale('image', width=100, height=80)
        size_low_quality = img_low_quality.size
        # data should be smaller at lower quality
        self.assertTrue(size_high_quality > size_low_quality)

    def testScaleThatCausesErrorsCanBeSuppressed(self):
        field = self.image.getField('image')
        field.swallowResizeExceptions = False
        self.assertRaises(
            Exception, self.adapter.scale, 'image', width=-1, height=-1)
        # scaling exceptions should be "swallowed" when set on the field...
        field.swallowResizeExceptions = True
        self.assertEqual(self.adapter.scale('image', width=-1, height=-1),
                         None)

    def testGetAvailableSizes(self):
        assert self.adapter.getAvailableSizes('image') == {'foo': (60, 60)}

    def testGetImageSize(self):
        assert self.adapter.getImageSize('image') == (200, 200)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)



from unittest import defaultTestLoader
from unittest import TestCase
from zope.component import provideAdapter
from zope.interface.verify import verifyClass
from zope.publisher.interfaces import IPublishTraverse
from plone.app.imaging.interfaces import IImageScaleHandler
from plone.app.imaging.traverse import ImageTraverser
from plone.app.imaging.traverse import DefaultImageScaleHandler
from Products.Archetypes.atapi import ImageField


data_marker = object()
fallback_marker = object()


class BaseMockField:

    def getAvailableSizes(self, context):
        return dict(mini=1, maxi=2)

    def getScale(self, context, scale):
        return data_marker


class MockField(BaseMockField, ImageField):

    pass


class MockContext:

    def Schema(self):
        return self

    def get(self, key):
        return getattr(self, key, None)


class TraverseTests(TestCase):

    def setUp(self):
        ImageTraverser.org_fallback = ImageTraverser.fallback
        ImageTraverser.fallback = lambda *args: fallback_marker
        provideAdapter(DefaultImageScaleHandler, (MockField,), IImageScaleHandler)

    def tearDown(self):
        ImageTraverser.fallback = ImageTraverser.org_fallback
        del ImageTraverser.org_fallback

    def testInterface(self):
        self.assertTrue(verifyClass(IPublishTraverse, ImageTraverser))

    def testUnknownField(self):
        traverser = ImageTraverser(MockContext(), None)
        self.assertTrue(traverser.publishTraverse(None, 'missing') is fallback_marker)

    def testWrongFieldType(self):
        context = MockContext()
        context.field = BaseMockField()
        traverser = ImageTraverser(context, None)
        self.assertTrue(traverser.publishTraverse(None, 'field') is fallback_marker)

    def testCorrectFieldType(self):
        context = MockContext()
        context.field = MockField()
        traverser = ImageTraverser(context, None)
        self.assertTrue(traverser.publishTraverse(None, 'field') is data_marker)

    def testFullImage(self):
        context = MockContext()
        context.field = MockField()
        traverser = ImageTraverser(context, None)
        self.assertTrue(traverser.publishTraverse(None, 'field') is data_marker)

    def testUnknownScale(self):
        context = MockContext()
        context.field = MockField()
        traverser = ImageTraverser(context, None)
        self.assertTrue(traverser.publishTraverse(None, 'field_poster') is fallback_marker)

    def testKnownScale(self):
        context = MockContext()
        context.field = MockField()
        traverser = ImageTraverser(context, None)
        self.assertTrue(traverser.publishTraverse(None, 'field_mini') is data_marker)


def test_suite():
    return defaultTestLoader.loadTestsFromName(__name__)



from Products.Archetypes.interfaces import IBaseObject as IATBaseObject
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface, Attribute

_ = MessageFactory('plone')


class IImageScale(Interface):
    """ representation of a scaled image, providing access to its metdata
        and actual payload data """

    id = Attribute('An identifier uniquely identifying this scale')
    width = Attribute('The pixel width of the image.')
    height = Attribute('The pixel height of the image.')
    url = Attribute('Absolute URL for this image.')
    mimetype = Attribute('The MIME-type of the image.')
    data = Attribute('The image data.')
    size = Attribute('The size of the image data in bytes.')
    filename = Attribute('The filename used for downloads.')


class IStableImageScale(Interface):
    """ Marker for image scales when accessed with a UID-based URL.

    These can be cached forever using the plone.stableResource ruleset.
    """


class IImageScaleFactory(Interface):
    """ adapter for image fields that allows generating scaled images """

    def create(context, **parameters):
        """ generate an image scale and return a tuple containing a
            representation of the actual image scale data (i.e. a string or
            file-like object) as well as the image's format and dimensions.
            for convenience, this happens to match the return value of
            `scaleImage`, but makes it possible to use different storages,
            i.e. zodb blobs """


class IImageScaling(Interface):
    """ adapter use for generating (and storing) image scales """

    def scale(fieldname=None, scalename=None, **parameters):
        """ retrieve a scale based on the given name or set of parameters.
            the parameters can be anything supported by `scaleImage` and
            would usually consist of at least a width & height.  returns
            either an object implementing `IImageScale` or `None` """

    def tag(fieldname=None, scalename=None, **parameters):
        """ returns a tag for a scale """

    def getAvailableSizes(fieldname=None):
        """ returns a dictionary of scale name => (width, height) """

    def getImageSize(fieldname=None):
        """ returns the original image size, a tuple of (width, height) """

    def getInfo(fieldname=None, scalename=None, **parameters):
        """ returns metadata for the requested scale from the storage """


class IImageScaleHandler(Interface):
    """ handler for retrieving scaled versions of an image """

    def getScale(instance, scale):
        """ return scaled and aq-wrapped version for given image data """

    def createScale(instance, scale, width, height, data=None):
        """ create & return a scaled version of the image as retrieved
            from the field or optionally given data """


class IBaseObject(IATBaseObject):
    """ marker interface used to be able to avoid having to use
        `overrides.zcml` to register our version of the traversal adapter """
